/**
 * @file Advance Form js widget Controller
 * @author Hussein Arian <hossein@puzzley.ir>
 * @version 1.1.1
 */
(function () {
	'use strict';

	angular.module('ngApp').requires.push(
		'formlyIonic',
		'mgo-angular-wizard',
		'ngMessages',
		'ngFileUpload',
		'ionic-datepicker',
		'leaflet-directive',
		'ionic-ratings'
	);

	angular
		.module('ngApp')
		.controller('advanceFormCtrl', advanceFormController)
		.controller('formListController', formListController)
		.controller('ticketController', ticketController)
		.factory('$cordovaGeolocation', cordovaGeolocation)
		.directive("focusNextOnEnter", focusNextInput);

	advanceFormController.$inject = [
		'$scope',
		'$http',
		'$timeout',
		'$ionicScrollDelegate',
		'$ionicPopup',
		'localStorage',
		'WizardHandler',
		'$rootScope',
		'$ionicLoading',
		'$sce'
	];
	function advanceFormController(
		$scope,
		$http,
		$timeout,
		$ionicScrollDelegate,
		$ionicPopup,
		localStorage,
		WizardHandler,
		$rootScope,
		$ionicLoading,
		$sce
	) {
		var vm = this;
		vm.fields = {};
		vm.steps_length = 0;
		vm.conditions = {};
		vm.termsFields = {};
		vm.prices = {};
		vm.hideStep = [];
		vm.currentStep = 0;
		vm.paymentStepActive = '';
		vm.coupon = false;
		vm.calculatePrice = calculatePrice;

		vm.options = {
			data:{
				showFormList: false,
				mode: "form",
				prepayment: {
					active: false,
					type: 'persentage',
					value: 0
				}
			}
		};
		vm.showFormList = false;
		// funcation assignment
		vm.onSubmit = onSubmit;
		vm.checkCoupon = checkCoupon;

		vm.model = {
			totalPrice: 0,
			discount: 0,
			finalPrice: 0,
			tax: 0
		};

		// get form data on sl init
		$scope.$watch('sl', function () {
			getForm();
		});

		/**
		 * get form field data and conditions
		 * send local form for get unread ticket count
		 */
		function getForm() {
			$ionicLoading.show();
			$http.get(url + '/api/v1/widget/get/advance-form/getForm?sl=' + $scope.sl + localFormToUrl())
				.success(function (result) {
					if (result.success) {
						vm.fields = result.payload.data.fields;
						vm.originalFields = angular.copy(vm.fields);
						vm.conditions = result.payload.data.conditions;
						vm.options = result.payload.options;
						initialConditons();
						$timeout(function () {
							vm.paymentmode = (vm.options.data.mode == 'payment');
							vm.showFormList = (result.payload.options.data.mode == 'payment' || result.payload.options.data.mode == 'form') && result.payload.options.data.showFormList;
							// calculate step length
							if (typeof Object.keys == "function") {
								vm.steps_length = Object.keys(result.payload.data.fields).length;
							} else {
								for (var step in result.payload.data.fields)
									vm.steps_length++;
							}
							vm.model.prepaymentselect = result.payload.options.data.prepayment.values[0];
							vm.options.data.payChequeContent = $sce.trustAsHtml(result.payload.options.data.payChequeContent);
						}, 100);
						$rootScope['unread_tickets' + $scope.formID] = vm.options.data.unread_tickets;
						//$scope.$watch('vm.model', checkConditions, true);
					} else {
						showWarning(result.message);
					}
					$ionicLoading.hide();
				})
				.error(function () {
					checkYourInternet();
					$ionicLoading.hide();
				});
		}

		function localFormToUrl() {
			var localForm = localStorage.all('form' + $scope.formID);
			var result = '';
			for (var index = 0; index < localForm.length; index++) {
				result += '&localform[]=' + localForm[index];
			}
			return result;
		}

		function initialConditons() {
			// check conditions terms and init termsFields
			// termsFields are fields that triger conditions
			var calculatepriceTerms = [];
			for (var c = 0; c < vm.conditions.length; c++) {
				var condition = vm.conditions[c];
				condition.index = c;
				for (var i = 0; i < condition.terms.length; i++) {
					if (condition.terms[i].field) {
						if (typeof vm.termsFields[condition.terms[i].field] != 'object')
							vm.termsFields[condition.terms[i].field] = [];
						vm.termsFields[condition.terms[i].field].push(condition.index);
					}
				}
				for (var ai = 0; ai < condition.actions.length; ai++) {
					if (condition.actions[ai].type == 'calculateprice') {
						calculatepriceTerms.push(condition.index);
					}
				}
			}
			// set watcher for termsFields to check is it pass conditons terms? (for any change)
			for (var fieldKey in vm.termsFields) {
				(function (fieldKey) {
					$scope.$watch('vm.model.' + fieldKey, function (newValue, oldValue, scope) {
						for (var i = 0; i < vm.termsFields[fieldKey].length; i++) {
							if (vm.model.checkTermsId(vm.termsFields[fieldKey][i])) {
								executeAction(vm.conditions[vm.termsFields[fieldKey][i]]);
							} else if (calculatepriceTerms.indexOf(vm.termsFields[fieldKey][i]) !== -1) {
								calculatePrice();
							}
						}
					}, true);
				})(fieldKey);
			}
		}

		vm.model.checkTermsId = function (index) {
			if (typeof index == "undefined")
				return false;
			return checkTerms(vm.conditions[index]);
		};

		// watch prices array change for re calculate total price
		$scope.$watch('vm.prices', calculatePrice, true);

		// watch totalprice to set payment step
		$scope.$watch('vm.paymentStepActive', function () {
			if (vm.paymentStepActive === true) {
				vm.steps_length++;
			} else if (vm.paymentStepActive === false) {
				vm.steps_length--;
			}
		}, true);

		/**
		 * calculate total price function
		 * first calculate passed and active steps
		 * then check price array to check conditions and add amuont
		 * at the end calculate tax and prepay amuont
		 * run on step change or conditions terms change
		 */
		function calculatePrice() {
			vm.model.totalPrice = vm.model.finalPrice = 0;
			var tmpTotalPrice = 0;
			var j = 0;
			for (var i in vm.fields) {
				if (!(typeof vm.hideStep[j] == "function" && vm.hideStep[j]()) && j <= vm.currentStep) {
					vm.model.totalPrice += parseFloat(vm.fields[i].price);
				}
				j++;
			}
			for (var key in vm.prices) {
				if (vm.prices[key].conditionIndex !== false) {
					if (checkTermsForPay(vm.prices[key].conditionIndex))
						vm.model.totalPrice += eval(prepareCalculation(vm.prices[key].price));
				} else {
					vm.model.totalPrice += parseInt(vm.prices[key].price);
				}
			}

			vm.paymentStepActive = vm.model.totalPrice > 0;
			// $timeout(function () {
			// 	vm.model.totalPrice = newValue;
			// });
			if (vm.options.data.prepayment.active) {
				vm.model.prepayment = calculatePrepayment(vm.model.totalPrice);
				tmpTotalPrice = parseInt(vm.model.prepayment);
			} else {
				tmpTotalPrice = parseInt(vm.model.totalPrice);
			}

			if (vm.coupon) {
				vm.model.discount = calculateCoupon(tmpTotalPrice);
				tmpTotalPrice -= vm.model.discount;
			}
			vm.model.tax = tmpTotalPrice * parseFloat(vm.options.data.tax) / 100;
			vm.model.finalPrice = tmpTotalPrice + vm.model.tax;
		}

		function prepareCalculation(str) {
			return str.replace(/\{([^\}]*)\}/gim, function (match, key) {
				return 'parseFloat(vm.model.' + key + ' || 0)';
			});
		}

		function calculatePrepayment(totalPrice) {
			if (vm.options.data.prepayment.type == 'persentage') {
				return totalPrice * vm.options.data.prepayment.value / 100;
			} else if (vm.options.data.prepayment.type == 'fixed') {
				return vm.options.data.prepayment.value;
			} else if (vm.options.data.prepayment.type == 'multipersentage') {
				return totalPrice * vm.model.prepaymentselect / 100;
			}
			return vm.model.prepayment;
		}

		function calculateCoupon(finalPrice) {
			if (vm.coupon.type == 'percentage') {
				return finalPrice * parseInt(vm.coupon.value) / 100;
			} else if (vm.coupon.type == 'fixed') {
				return parseInt(vm.coupon.value);
			}
			return 0;
		}

		// check term and check field not hided
		function checkTermsForPay(conditionsIndex) {
			var condition = vm.conditions[conditionsIndex];
			if (condition)
				if (condition.link == 'or') {
					for (var t = 0; t < condition.terms.length; t++) {
						if (checkTerm(condition.terms[t]) && findField(condition.terms[t].field).hide !== true) {
							return true;
						}
					}
				} else if (condition.link == 'and') {
					var execute = true;
					for (var t = 0; t < condition.terms.length; t++) {
						if (!(checkTerm(condition.terms[t]) && findField(condition.terms[t].field).hide !== true)) {
							execute = false;
						}
					}
					return execute;
				}
			return false;
		}

		$scope.$on('wizard:stepChanged', function (event, data) {
			$timeout(function() {
				$ionicScrollDelegate.$getByHandle('mainContent').scrollTop();
				$scope.$broadcast('scroll.resize');
			});
			$timeout(function() {
				$ionicScrollDelegate.$getByHandle('mainContent').resize();
			}, 300);
			vm.currentStep = data.step.wzData.step;
			calculatePrice();
		});

		vm.exitValidation = function (form) {
			return form && form.$valid;
		};
		
		vm.beforeNextStep = function (form) {
			form.$setSubmitted();
			if (form && form.$invalid) {
				showWarning('فرم را کامل و معتبر پر کنید');
			}
		};


		function checkTerms(condition) {
			if (condition)
				if (condition.link == 'or') {
					for (var t = 0; t < condition.terms.length; t++) {
						if (checkTerm(condition.terms[t])) {
							return true;
						}
					}
				} else if (condition.link == 'and') {
					var execute = true;
					for (var t = 0; t < condition.terms.length; t++) {
						if (!checkTerm(condition.terms[t])) {
							execute = false;
						}
					}
					return execute;
				}
			return false;
		}

		function checkTerm(term) {
			switch (term.operator) {
				case 'equals':
					if (typeof vm.model[term.field] == "object")
						return vm.model[term.field].hasOwnProperty(term.value) && vm.model[term.field][term.value];
					return vm.model[term.field] == term.value;
				case 'notEquals':
					if (typeof vm.model[term.field] == "object")
						return !(vm.model[term.field].hasOwnProperty(term.value) && vm.model[term.field][term.value]);
					return vm.model[term.field] != term.value;
				case 'greaterThan':
					return vm.model[term.field] > term.value;
				case 'lessThan':
					return vm.model[term.field] < term.value;
				case 'fill':
					return vm.model[term.field] != '';
				default:
					return false;
					break;
			}
		}

		function executeAction(condition) {
			for (var a = 0; a < condition.actions.length; a++) {
				var action = condition.actions[a];
				switch (action.type) {
					case 'templateOptions':
						findField(action.field).templateOptions[action.property] = action.value;
						break;
					case 'setValue':
						vm.model[action.field] = action.value;
						break;
					case 'hideExpression':
						findField(action.field).hideExpression = 'model.checkTermsId(' + condition.index + ')';
						break;
					case 'showExpression':
						findField(action.field).hideExpression = '!model.checkTermsId(' + condition.index + ')';
						break;
					case 'hideExpressionMulti':
						for (var i = 0; i < action.fields.length; i++) {
							findField(action.fields[i]).hideExpression = 'model.checkTermsId(' + condition.index + ')';
						}
						break;
					case 'showExpressionMulti':
						for (var i = 0; i < action.fields.length; i++) {
							findField(action.fields[i]).hideExpression = '!model.checkTermsId(' + condition.index + ')';
						}
						break;
					case 'required':
						if (typeof (findField(action.field).expressionProperties) != "object") {
							findField(action.field).expressionProperties = {};
						}
						findField(action.field).expressionProperties["templateOptions.required"] = 'model.checkTermsId(' + condition.index + ')';
						break;
					case 'unrequired':
						if (typeof (findField(action.field).expressionProperties) != "object") {
							findField(action.field).expressionProperties = {};
						}
						findField(action.field).expressionProperties["templateOptions.required"] = '!model.checkTermsId(' + condition.index + ')';
						break;
					case 'disabled':
						if (typeof (findField(action.field).expressionProperties) != "object") {
							findField(action.field).expressionProperties = {};
						}
						findField(action.field).expressionProperties["templateOptions.disabled"] = 'model.checkTermsId(' + condition.index + ')';
						break;
					case 'hideStep':
						vm.hideStep[action.stepIndex -1] = function () {
							return vm.model.checkTermsId(condition.index)
						};
						break;
					case 'copyFieldValue':
						vm.model[action.target] = vm.model[action.field];
						break;
					case 'calculation':
						var equation = prepareCalculation(action.equation);
						vm.model[action.field] = eval(equation);
						break;
					case 'calculateprice':
						vm.prices[action.id] = {
							price: action.equation,
							conditionIndex: condition.index,
							rand: Math.random()
						}
						break;
					default:
						break;
				}
			}
		}

		function findField(key) {
			for (var step in vm.fields) {
				if (vm.fields[step].items.hasOwnProperty(key))
					return vm.fields[step].items[key];
			}
		}

		// function definition
		function onSubmit() {
			// console.log('onSubmit');
			// if (vm.options.data.mode != 'payment')
				for (var form in vm.forms)
					if (vm.forms[form].$invalid)
						return;
			var fd = new FormData();
			for (var key in vm.model) {
				if (typeof vm.model[key] == "object") {
					var field = findField(key);
					if (field["type"] == 'file') {
						fd.append(key, vm.model[key]);
					} else {
						fd.append(key, JSON.stringify(vm.model[key]));
					}
				} else if (typeof vm.model[key] != "function") {
					fd.append(key, vm.model[key]);
				}
			}
		
			fd.append('sl', $scope.sl);
			
			$http.post(url + '/api/v1/widget/post/advance-form/postForm', fd, {
				withCreadential: true,
				headers: {
					'Content-Type': undefined,
				},
				transformRequest: angular.identity
			})
				.success(function (result) {
					if (result.success) {
						localStorage.addnew(result.payload.transaction_id, 'form' + $scope.formID);
						$ionicPopup.alert({
							title: 'موفق',
							template: result.message,
							okText: "بستن"
						}).then(function () {
							if (result.payload.redirect_url != '')
								redirect(result.payload.redirect_url);
							if (vm.options.data.mode == 'ticket')
								window.history.back();
						});
						// save tansaction_id for later follow up
						// rest form
						vm.options.resetModel();
						WizardHandler.wizard('WizardHandler').reset();
					}
					//return response;
				})
				.error(function () {
					checkYourInternet();
					return response;
				});
		}

		function redirect(url) {
			var target = '_system';
			if (isDevice) {
				inAppBrowserRef = window.open(url, target, "location=yes&clearcache=yes&zoom=no&toolbar=no,status=no,titlebar=no");
				// with(inAppBrowserRef) {
				// 	addEventListener('loadstart', function (event) {
				// 		if (event.url.search('getCallback') != -1) {
							
				// 		}
				// 	});
				// }
			} else {
				window.open(url, target, "location=yes&clearcache=yes&zoom=no&toolbar=no,status=no,titlebar=no");
			}
		}

		/**
		 * check Coupon code and apply it to total price
		 *
		 * @param string code
		 */
		function checkCoupon(code) {
			$http.post(url + '/api/v1/widget/post/advance-form/checkCoupon', {
				code: code,
				sl: $scope.sl
			})
				.success(function (result) {
					if (result.success) {
						showSuccess(result.message, 2500);
						vm.coupon = result.payload.coupon;
						calculatePrice();
					} else {
						showError(result.message);
					}
					//return response;
				})
				.error(function () {
					checkYourInternet();
					return response;
				});
		}

	}

	/** 
	 * form List page Controller
	 */
	formListController.$inject = ['$scope', '$http', 'localStorage'];
	function formListController($scope, $http, localStorage) {
		var vm = this;
		vm.search = search;
		vm.loadMore = loadMore;
		vm.refresh = refresh;
		vm.noMoreItemsAvailable = true;
		vm.infiniteScrollLoading = false;
		vm.message = '';
		var length = 20;

		$scope.$watch('sl', function () {
			$scope.localForm = localStorage.all('form' + $scope.formID);
			getformlist();
		});
		
		function getformlist() {
			$http.post(url + '/api/v1/widget/post/advance-form/postFormList', {
				localform: $scope.localForm,
				sl: $scope.sl,
				length: length
			})
				.success(function (result) {
					if (result.success) {
						vm.orginalFormlist = result.payload.formlist;
						vm.formlist = vm.orginalFormlist;
						if (result.payload.formlist.length == length) {
							vm.noMoreItemsAvailable = false;
						}
					}
					$scope.$broadcast('scroll.refreshComplete');
				})
				.error(function () {
					checkYourInternet();
					$scope.$broadcast('scroll.refreshComplete');
				});
		}

		function refresh() {
			$scope.localForm = localStorage.all('form' + $scope.formID);
			getformlist();
		}

		function search() {
			$http.post(url + '/api/v1/widget/post/advance-form/postFormList', {
				sl: $scope.sl,
				q: vm.searchText
			})
				.success(function (result) {
					if (result.success) {
						vm.formlist = result.payload.formlist;
						vm.message = result.message;
					} else {
						showWarning(result.message);
					}
				})
				.error(function () {
					checkYourInternet();
				});
		}
		
		// watch search text for show last fetched formlist if its blank
		$scope.$watch('vm.searchText', function (newValue) {
			if (newValue == '') {
				vm.formlist = vm.orginalFormlist;
			}
		});

		function loadMore() {
			if (!vm.noMoreItemsAvailable && !vm.searchMode) {
				vm.noMoreItemsAvailable = true;
				var start = vm.formlist.length;
				if (isOnline()) {
					vm.infiniteScrollLoading = true;
					var data = {
						sl: $scope.sl,
						start: start,
						length: length
					};
					if (typeof vm.searchText != "undefined" && vm.searchText != "") {
						data.q = vm.searchText;
					} else {
						data.localform = $scope.localForm;
					}
					$http({
							method: 'POST',
							noLoading: true,
							url: url + '/api/v1/widget/post/advance-form/postFormList',
							data: data
						})
						.success(function (result) {
							if (result.success) {
								for (var i = 0; i < result.payload.formlist.length; i++) {
									vm.formlist.push(result.payload.formlist[i]);
								}
								vm.noMoreItemsAvailable = result.payload.formlist.length < length;
							}
							
							vm.infiniteScrollLoading = false;
							$scope.$broadcast('scroll.infiniteScrollComplete');
						})
						.error(function () {
							console.error('ERR', err.status);
							vm.infiniteScrollLoading = false;
							$scope.$broadcast('scroll.infiniteScrollComplete');
						});
				}
			} else {
				return;
			}
		}
	}

	/** 
	 * single ticket controller
	*/
	ticketController.$inject = ['$rootScope', '$scope', '$http'];
	function ticketController($rootScope, $scope, $http) {
		var vm = this;
		vm.ticket = {};
		vm.sendTicket = sendTicket;
		vm.usertickets = [];
		vm.sendRates = sendRates;
		var rates = [];
		vm.rateOptions = {
			readOnly: false,
			rating: 0,
			minRating: 0,
			callback: function (rating, index) {
				rates[index] = rating;
			}
		};

		$scope.$watch('transaction_id', function () {
			$rootScope['ticket' + $scope.transaction_id] = 'visited';
			$rootScope['unread_tickets' + $scope.formID] -= parseInt($scope.unread_ticket_count);
		});

		function sendTicket(valid) {
			if (valid) {
				var fd = new FormData();
				for (var key in vm.ticket) {
					fd.append(key, vm.ticket[key]);
				}
				fd.append('sl', $scope.sl);

				$http.post(url + '/api/v1/widget/post/advance-form/postTicket', fd, {
						withCreadential: true,
						headers: {
							'Content-Type': undefined,
						},
						transformRequest: angular.identity
					})
					.success(function (result) {
						if (result.success) {
							vm.ticket = {
								title: '',
								content: '',
								attach: ''
							};
							showToast('تیکت شما با موفقیت ثبت شد');
							vm.usertickets.push(result.payload.ticket);
							$scope.fm.$setPristine();
						} else {
							showToast(result.message);
						}
					})
					.error(function () {
						showToast('خطا در ارسال، اتصال اینترنت خود را بررسی کنید');
					});
			} else {
				showToast('فرم را کامل و معتبر وارد کنید');
				return;
			}
		}

		function sendRates() {
			$http.post(url + '/api/v1/widget/post/advance-form/postSetRates', {
				sl: $scope.sl,
				transaction_id: $scope.transaction_id,
				rates: rates
			})
				.success(function (result) {
					showToast(result.message);
				})
				.error(function () {
					showToast('خطا در ارسال، اتصال اینترنت خود را بررسی کنید');
				});
		}
	}

	/** 
	 * cordovaGeolocation factory
	 * inhert from ngCordova project
	*/
	cordovaGeolocation.$inject = ['$q'];
	function cordovaGeolocation($q) {
		return {
			getCurrentPosition: function (options) {
				var q = $q.defer();

				navigator.geolocation.getCurrentPosition(function (result) {
					q.resolve(result);
				}, function (err) {
					q.reject(err);
				}, options);

				return q.promise;
			},

			watchPosition: function (options) {
				var q = $q.defer();

				var watchID = navigator.geolocation.watchPosition(function (result) {
					q.notify(result);
				}, function (err) {
					q.reject(err);
				}, options);

				q.promise.cancel = function () {
					navigator.geolocation.clearWatch(watchID);
				};

				q.promise.clearWatch = function (id) {
					navigator.geolocation.clearWatch(id || watchID);
				};

				q.promise.watchID = watchID;

				return q.promise;
			},

			clearWatch: function (watchID) {
				return navigator.geolocation.clearWatch(watchID);
			}
		};
	}


	function focusNextInput() {
		return {
			restrict: "A",
			link: function ($scope, element) {
				element.on("keypress", function (e) {
					if (e.keyCode == 13) {
						var nextInput = $(':input:not(:disabled):not(button):eq(' + ($(':input:not(:disabled):not(button)').index(element) + 1) + ')');
						if (nextInput.length) {
							nextInput.focus();
						} else {
							Keyboard.hide();
						}
					}
				});
			}
		};
	}
})();
