angular.module('ngApp').controller('NewsCtrl', function ($scope, $rootScope, $ionicModal, $http, localStorage, $ionicScrollDelegate, $sce, $timeout) {

    $scope.news = [];
    $scope.noMoreItemsAvailable = true;
    $scope.storage = '';

    // Wait until the sl var is init
    $scope.$watch('sl', function () {
        $scope.showNews(false);
        $scope.getSliders(false);
    });

    $scope.showNews = function (refresh) {
        var data = localStorage.all($scope.hash);
        if (data) {
            $scope.news = data;
        }
        if (isOnline()) {
            if (!refresh) {
                $scope.loading = true;
            }
            $http({
                    method: 'GET',
                    noLoading: true,
                    url: url + '/api/v1/widget/route/news/getNews/' + $scope.sl + '?length=6'
                })
                .success(function (data) {
                    $scope.$broadcast('scroll.refreshComplete');

                    $scope.storage = '';
                    localStorage.save(data, $scope.hash);
                    $scope.news = data;
                    // setTimeout(function() {
                    if (data.length == 6) {
                        $scope.noMoreItemsAvailable = false;
                    }
                    // }, 100);
                    $scope.loading = false;
                })
                .error(function () {
                    $scope.$broadcast('scroll.refreshComplete');
                    data = localStorage.all($scope.hash);
                    $scope.storage = 'خطا در اتصال به سرور';
                    $scope.news = data;
                    $scope.loading = false;
                });
        } else {
            $scope.loading = false;
            showToast("اتصال اینترنت خود را بررسی کنید");
        }
    };

    $scope.getSliders = function (refresh) {
        $scope.sliderLoading = true;
        $http({
                method: 'GET',
                async: false,
                noLoading: false,
                url: url + '/api/v1/widget/route/news/getSliders/' + $scope.sl
            })
            .success(function (slider) {
                if (slider.slides) {
                    $scope.slider = slider;

                    var loop = $scope.slider.effect == 'cube' ? false : true;
                    $scope.slideroptions = {
                        preventClicks: false,
						preventClicksPropagation: false,
						autoplayDisableOnInteraction: false,
                        loop: loop,
                        speed: 400,
                        autoplay: $scope.slider.auto_play * $scope.slider.duration * 1000,
                        effect: $scope.slider.effect
                    };
                }
                if (slider.start_content) {
                    $scope.start_content = $sce.trustAsHtml(slider.start_content);
                }

                $timeout(function () {
                    $scope.sliderLoading = false;
                }, 5000);
            })
            .error(function () {
                $scope.sliderLoading = false;
            });
    };

    $scope.openModal = function (index) {
        $ionicModal.fromTemplateUrl('news-modal.html', {
            scope: $scope,
            animation: 'slide-in-up',
            hardwareBackButtonClose: true
        }).then(function (modal) {
            $scope.modal = modal;
            $scope.image = $scope.news[index].image;
            $scope.fetureImage = false;
            if ($scope.image != '') $scope.fetureImage = true;
            $scope.caption = $scope.news[index].title;
            $scope.content = $sce.trustAsHtml($scope.news[index].description);
            $scope.shareUrl = $scope.news[index].shareUrl;
            $scope.modal.show();
            $rootScope.openModal = $scope.modal;
            $ionicScrollDelegate.$getByHandle('modal').scrollTop();
        });       
    };

    $scope.closeModal = function () {
        $scope.modal.remove();
    };

    $scope.share = function(event) {
        var titleElem = document.getElementsByClassName('share-title');
        var messageElem = document.getElementsByClassName('share-message');
        var imageElem = document.getElementsByClassName('share-image');
        var urlElem = document.getElementsByClassName('share-url');
        var shareObj = {};
        if (titleElem) {
            titleElem = titleElem[0];
            shareObj.subject = titleElem.textContent;
        }
        if (messageElem) {
            messageElem = messageElem[0];
            shareObj.message = shareObj.subject ?
                 shareObj.subject + "\n" + messageElem.textContent :
                messageElem.textContent;
        }
        if (imageElem) {
            imageElem = imageElem[0];
            shareObj.files = [imageElem.getAttribute('src')];
        }
        if (urlElem) {
            urlElem = urlElem[0];
            if (urlElem.dataset.shareUrl) {
                shareObj.url = urlElem.dataset.shareUrl;
            }
        }
        if (isDevice) {
            window.plugins.socialsharing.available(function (isAvailable) {
                if (isAvailable) {
                    window.plugins.socialsharing.shareWithOptions(
                        shareObj,
                        function () {
                            // callback on succeed
                        },
                        function () {
                            // callback on failed
                        }
                    );
                }
            });
        } else {
            $rootScope.webAppShare(event, shareObj.subject, shareObj.url);
        }
    };

    $scope.doRefresh = function () {
        $scope.showNews(true);
    };

    $scope.loadMore = function () {
        if (!$scope.noMoreItemsAvailable) {
            $scope.noMoreItemsAvailable = true;
            var start = $scope.news.length;
            if (isOnline()) {
                $scope.infiniteScrollLoading = true;
                $http({
                        method: 'GET',
                        noLoading: true,
                        url: url + '/api/v1/widget/route/news/getNews/' + $scope.sl + '/' + start + '?length=6'
                    })
                    .success(function (data) {
                        $scope.noMoreItemsAvailable = data.length < 6;
                        if (data.length == 0 || data.found == 0) {
                            $scope.noMoreItemsAvailable = true;
                        } else {
                            for (var i = 0; i < data.length; i++) {
                                $scope.news.push(data[i]);
                            }
                        }
                        $scope.infiniteScrollLoading = false;
                        $scope.$broadcast('scroll.infiniteScrollComplete');
                    })
                    .error(function () {
                        console.error('ERR', err.status);
                        $scope.infiniteScrollLoading = false;
                        $scope.$broadcast('scroll.infiniteScrollComplete');
                    });
            }

        }
    };

    // Cleanup the modal when we're done with it!
    $scope.$on('$destroy', function () {
        $scope.modal.remove();
    });
    
    // Execute action on hide modal
    $scope.$on('modal.hidden', function () {
        $scope.modal.remove();
        // document.querySelectorAll('video').forEach(vid => vid.pause());
        // document.querySelectorAll('audio').forEach(vid => vid.pause());
    });
})