angular.module('ngApp.services', [])
/**
 * services for storage object as json in window.localstorage
 * @type  {factory service}
 */
.factory('localStorage', function() {
  return {
	all: function(key) {
	  var data = window.localStorage[key];
	  if(data) {
		return angular.fromJson(data);
	  }
	  return {};
	},
	save: function(data,key) {
	  window.localStorage[key] = angular.toJson(data);
	},
	clear: function(key) {
	  window.localStorage.removeItem(key);
	},
	get: function(key,id) {
	  var data = window.localStorage[key];
	  if(data) {
		data = angular.fromJson(data);
		return data[id];
	  }
	  return {};
    },
	add: function(data,key) {
		var old_data = window.localStorage[key];
		if(old_data) {
			old_data = angular.fromJson(old_data);
			data = angular.merge(old_data, data);
		}
		window.localStorage[key] = angular.toJson(data);
	},
	addnew: function(data,key) {
		var storage = window.localStorage[key];
		if(storage) {
			storage = angular.fromJson(storage);
		} else {
            storage = [];
        }
        storage.push(data);
		window.localStorage[key] = angular.toJson(storage);
	},
	remove: function(key,id) {
		var storage = window.localStorage[key];
		if(storage) {
            storage = angular.fromJson(storage);
            delete storage[id];
            window.localStorage[key] = angular.toJson(storage);
		}
	}
  }
})
/**
 * services to convert gregorian string date to persian date array
 * @type  {factory service}
 */
.factory('gregorianToJulian', function() {
    function mod(a, b) {
        return a - ~~(a / b) * b
    }

    function leapGregorian(year) {
        return ((year % 4) === 0) &&
            (!(((year % 100) === 0) && ((year % 400) !== 0)));
    }

    // GREGORIAN_TO_JD  --  Determine Julian day number from Gregorian calendar date
    function gregorianToJulian(year, month, day) {
        var pad;
        if (month <= 2) {
            pad = 0;
        } else if (leapGregorian(year)) {
            pad = -1;
        } else {
            pad = -2;
        }

        return (1721425.5 - 1) +
            (365 * (year - 1)) +
            Math.floor((year - 1) / 4) +
            (-Math.floor((year - 1) / 100)) +
            Math.floor((year - 1) / 400) +
            Math.floor((((367 * month) - 362) / 12) + (pad + day));
    }

    function persianToJulian(year, month, day) {
        var epbase = year - ((year >= 0) ? 474 : 473);
        var epyear = 474 + mod(epbase, 2820);

        return day +
            ((month <= 7) ?
            ((month - 1) * 31) :
            (((month - 1) * 30) + 6)
            ) +
            Math.floor(((epyear * 682) - 110) / 2816) +
            ((epyear - 1) * 365) +
            (Math.floor(epbase / 2820) * 1029983) + (1948320.5 - 1);
    }

    function julianToPersian(jd) {
        var njd = Math.floor(jd) + 0.5;
        var depoch = njd - persianToJulian(475, 1, 1);
        var cycle = Math.floor(depoch / 1029983);
        var cyear = mod(depoch, 1029983);
        var ycycle;
        if (cyear === 1029982) {
            ycycle = 2820;
        } else {
            var aux1 = Math.floor(cyear / 366);
            var aux2 = mod(cyear, 366);
            ycycle = Math.floor(((2134 * aux1) + (2816 * aux2) + 2815) / 1028522) +
            aux1 + 1;
        }
        var year = ycycle + (2820 * cycle) + 474;
        if (year <= 0) {
            year -= 1;
        }
        var yday = (njd - persianToJulian(year, 1, 1)) + 1;
        var month = (yday <= 186) ? Math.ceil(yday / 31) : Math.ceil((yday - 6) / 30);
        var day = (njd - persianToJulian(year, month, 1)) + 1;

        return [year, month, day];
    }

    /**
     * Convert gregorian date To persian julian 
     * 
     * The functionality is based on great library https://github.com/arashm/JDate
     * 
     * @param  {dateString} dateString 
     */
    return function (dateString) {
        var date = new Date(dateString);
        return julianToPersian(gregorianToJulian(date.getUTCFullYear(), date.getUTCMonth() + 1, date.getUTCDate()));
    }
})
/**
 * conver string to standard date format for ng-model in input type date
 * @type  {directive}
 * @return  {intiger} standard date format
 */
.directive('stringToDate', function() {
    return {
        restrict : 'A',
        require: 'ngModel',
        link: function(scope, element, attrs, ngModel) {
            ngModel.$parsers.push(function(value) {
                return '' + value;
            });
            ngModel.$formatters.push(function(date) {
                return new Date(date);
            });
        }
    }
})
/**
 * String to number for ng-model in number input
 * @type  {directive}
 * @return  {intiger} standard number format
 */
.directive('stringToNumber', function() {
    return {
        restrict : 'A',
        require: 'ngModel',
        link: function(scope, element, attrs, ngModel) {
            ngModel.$parsers.push(function(value) {
                return '' + value;
            });
            ngModel.$formatters.push(function(value) {
                return parseFloat(value);
            });
        }
    };
})
/**
 * String to time for ng-model in time input
 * @type  {directive}
 * @return  {time} standard time format
 */
.directive('stringToTime', function() {
    return {
        restrict : 'A',
        require: 'ngModel',
        link: function(scope, element, attrs, ngModel) {
            ngModel.$parsers.push(function(value) {
                return '' + value;
            });
            ngModel.$formatters.push(function(value) {
                value = value.split(':');
                if (value[2] == undefined) {
                    value[2] = '00';
                }
                return new Date(1970,0,1, value[0], value[1], value[2]);
            });
        }
    };
})
/**
 * String to time for ng-model in time input
 * @type  {directive}
 * @return  {time} standard time format
 */
.directive('stringToBoolean', function() {
    return {
        restrict : 'A',
        require: 'ngModel',
        link: function(scope, element, attrs, ngModel) {
            ngModel.$parsers.push(function(value) {
                return '' + value;
            });
            ngModel.$formatters.push(function(value) {
                return value == '1' ||  value == 'true';
            });
        }
    };
})
.directive('backButton', function () {
    return {
        restrict: 'E',
        replace: true,
        controller: function($scope, $ionicHistory){
            $scope.canGoBack = false;
            if ($ionicHistory.backView()) {
                $scope.canGoBack = true;
            }
            $scope.goBack = function() {
                $ionicHistory.goBack();
            }
        },
        template: [
            '<button class="button back-buttonx buttons button-clear" ng-show="canGoBack" ng-click="goBack()">',
            '<i class="icon ion-arrow-left-c"></i>',
            '</button>',
        ].join('')
    };
})
.directive('hideTabs', function ($rootScope) {
    return {
        restrict: 'A',
        link: function ($scope, $el) {
            $rootScope.hideTabs = 'tabs-item-hide';
            $scope.$on('$destroy', function () {
                $rootScope.hideTabs = '';
            });
        }
    };
})
.directive('initBind', function($compile) {
    return {
        restrict: 'A',
        link : function (scope, element, attr) {
            console.log('link');
            attr.$observe('ngBindHtml',function(){
                console.log('compile');
                if(attr.ngBindHtml){
                    $compile(element[0].children)(scope);
                }
            })
        }
    };
})
.directive('noItem', function() {
    return {
        restrict: 'E',
        template: function (elem, attr) {
            return (
                '<div style="color: #b2b2b2; display: flex; felx: 1; justify-content: center;"><i style="margin-left: 5px; font-size: 17px;" class="icon ion-android-sad"></i>'+ attr.message +'</div>'
            );
        }
    }
})
.directive('focusMe', function($timeout) {
  return {
    link: function(scope, element, attrs) {

      $timeout(function() {
        element[0].focus(); 
      },750);
    }
  };
})
.directive("horizontalScrollFix", ["$timeout", "$ionicScrollDelegate", function ($timeout, $ionicScrollDelegate) {
	return {
		restrict: "A",
		link: function (scope, element, attrs) {
			var sv = $ionicScrollDelegate.$getByHandle(attrs.delegateHandle).getScrollView();
			var container = sv.__container;
			var originaltouchStart = sv.touchStart;
			var originalmouseDown = sv.mouseDown;
			var originaltouchMove = sv.touchMove;
			var originalmouseMove = sv.mouseMove;
			container.removeEventListener('touchstart', sv.touchStart);
			container.removeEventListener('mousedown', sv.mouseDown);
			document.removeEventListener('touchmove', sv.touchMove);
			document.removeEventListener('mousemove', sv.mousemove);
			sv.touchStart = function (e) {
				e.preventDefault = function () {}
				originaltouchStart.apply(sv, [e]);
			}
			sv.touchMove = function (e) {
				e.preventDefault = function () {}
				originaltouchMove.apply(sv, [e]);
			}
			sv.mouseDown = function (e) {
				e.preventDefault = function () {}
				originalmouseDown.apply(sv, [e]);
			}
			sv.mouseMove = function (e) {
				e.preventDefault = function () {}
				originalmouseMove.apply(sv, [e]);
			}
			container.addEventListener("touchstart", sv.touchStart, false);
			container.addEventListener("mousedown", sv.mouseDown, false);
			document.addEventListener("touchmove", sv.touchMove, false);
			document.addEventListener("mousemove", sv.mouseMove, false);
		}
	}
}])
.filter('toman',function() {
	return function (input) {
		if (input && lang('global.locale') == 'fa') {
			input = input.replace(/0/g, '۰').replace(/1/g, '۱').replace(/2/g, '۲').replace(/3/g, '۳').replace(/4/g, '۴').replace(/5/g, '۵').replace(/6/g, '۶').replace(/7/g, '۷').replace(/8/g, '۸').replace(/9/g, '۹');
			return input + ' ' + lang('global.toman');
		}
		return input;
	}
})
.filter('secondsToHMS',function() {
	return function(input) {
		if (input) {
            var date = new Date(0, 0, 0, 0, 0, input);
            var h = (date.getDay() * 24) + (date.getHours());
            h = h == 0 ? '' : h + ':';
            var m = (date.getMinutes());
            m = m == 0 ? '' : m + ':';
            var s = (date.getSeconds());
            s = s == 0 ? '' : s + ':';
            
            return (h + m + s).slice(0, -1);;
		}
	}
})
.filter('persianNumber',function() {
	return function(input) {
		if (input && lang('global.locale') == 'fa') {
			return input.toString().replace(/0/g,'۰').replace(/1/g,'۱').replace(/2/g,'۲').replace(/3/g,'۳').replace(/4/g,'۴').replace(/5/g,'۵').replace(/6/g,'۶').replace(/7/g,'۷').replace(/8/g,'۸').replace(/9/g,'۹');
        }
        return input;
	}
})
.filter('isNotEmptyStringAndDash', function() {
    return function (value) {
        return (
            (typeof value != 'undefined')        //undefined
            &&
            (value != null)                     //null
            &&
            (value != false)                    //false
            &&
            (value.length != 0)                 //empty
            &&
            (value != "")                       //empty
            &&
            (value.replace(/\s/g,"") != "")     //empty
            &&
            (/[^\s]/.test(value))              //empty
            &&
            (!/^\s*$/.test(value))               //empty
            &&
            (value.indexOf('-') == -1)               //dash
        );
    }
})
.filter('isEmptyString', function() {
    return function (value) {
        return (
            (typeof value == 'undefined')        //undefined
            ||
            (value == null)                     //null
            ||
            (value == false)                    //false
            ||
            (value.length == 0)                 //empty
            ||
            (value == "")                       //empty
            ||
            (value.replace(/\s/g,"") == "")     //empty
            ||
            (!/[^\s]/.test(value))              //empty
            ||
            (/^\s*$/.test(value))               //empty
        );
    }
});

function getValue(obj,i) {return obj[i]}
function lang(key) {
    try {
        var trans = key.split('.').reduce(getValue, _lang);
        return typeof trans != "undefined" ? trans : key;
    }
    catch(err) {
        return key;
    }
}
