function successPayVideoCommerce() {
    showSuccess(lang('video-commerce.payment_success'));
    window.location.reload();
}

function errorPayVideoCommerce() {
    showError(lang('video-commerce.your_payment_was_not_successful'));
}

angular.module('ngApp').requires.push('ionic-audio');
angular
    .module('ngApp')
    .controller('videoCommerceCtrl', function (
        $scope,
        $http,
        $ionicScrollDelegate,
        $location,
        localStorage,
        $stateParams,
        $rootScope,
        $ionicLoading,
        $ionicPopup,
        MediaManager,
        $timeout,
        $interval
    ) {
        $scope.isDevice = isDevice;
        $scope.config = $widgets.data.video_commerce;
        if (!isDevice) { // remove download link in web app if download_in_app is enable
            $scope.config.can_download_videos = $scope.config.can_download_videos && !$scope.config.download_in_app;
        }
        //$scope.togglePlayback = false;
        /**
         * NOTE: CURRENTLY NOT USED
         * ISSUE: #83
         * 
         * This function responsible to save current status of more items
         * to the local storage, it is neccessary for us to know what is 
         * the previous status of moreItems when going back to the previous
         * page.
         */
        $scope.resolveMoreItems = function (currentLocation) {
            if (currentLocation.indexOf('/video/') !== -1) {
                return;
            }
            var storageKey = $scope.initData.hash + '|moreItems|' + currentLocation;
            var oldValue = localStorage.all(storageKey);
            if (angular.equals(oldValue, {}) && $scope.moreItems != false) {
                $scope.moreItems = true;
                localStorage.save($scope.moreItems, storageKey);
            } else {
                localStorage.save($scope.moreItems, storageKey);
            }
        };

        /**
         * NOTE: CURRENTLY NOT USED
         * ISSUE: #83
         * 
         * This function responsible to save current status of next page
         * to the local storage, it is neccessary for us to know what is 
         * the previous status of nextPage when going back to the previous
         * page.
         */
        $scope.resolveNextPage = function (currentLocation) {
            if (currentLocation.indexOf('/video/') !== -1) {
                return;
            }
            var storageKey = $scope.initData.hash + '|nextPage|' + currentLocation;
            var oldValue = localStorage.all(storageKey);
            if (angular.equals(oldValue, {}) && $scope.nextPage != 0) {
                $scope.nextPage = 0;
                localStorage.save($scope.nextPage, storageKey);
            } else {
                localStorage.save($scope.nextPage, storageKey);
            }
        };

        $scope.refreshPaginationData = function () {
            $scope.moreItems = true;
            $scope.nextPage = 0;
            $scope.refresh = true;
        };

        // Init basic data in first loading
        $scope.landing = [];
        $scope.videos = [];
        $scope.video = localStorage.all('current-video');
        $scope.refreshPaginationData();

        // Lets fresh the init data when route was changed to avoid data cinflict
        //$scope.$on('$locationChangeStart', function ($event, next, current) {
            // console.log('route changed!');
            // $scope.refreshPaginationData(); <-------  Old code --------------------------|
            // $scope.resolveNextPage(current);  // -|                                         |---> But every thing is ok when this codes was commented
            //                                    |----> New solution to resolve issue #83 -|
            // $scope.resolveMoreItems(current); // -|
        //});

        $scope.onIndexPageInit = function (initData) {
            // console.log('onIndexPageInit');
            $scope.initData = initData;
            $scope.loadMainPageContents(false);
        };

        $scope.onVideoPageInit = function (initData) {
            // console.log('onVideoPageInit');
            $scope.initData = initData;
        };

        $scope.onVideoForSearchPageInit = function (initData) {
            $scope.initData = initData;
            $scope.video = {
                code: (initData.hash + '|' + $stateParams.id)
            };
            $scope.loadVideo(false);
        };

        $scope.onVideosPageInit = function (initData) {
            // console.log('onVideosPageInit');
            $scope.initData = initData;
            $scope.categories = localStorage.all('current-categories');
            $scope.slider = localStorage.all('current-slider');
            var loop = $scope.slider.effect == 'cube' ? false : true;
            $scope.slideroptions = {
                preventClicks: false,
                preventClicksPropagation: false,
                autoplayDisableOnInteraction: false,
                loop: loop,
                speed: 400,
                //autoHeight: true,
                autoplay: $scope.slider.duration * 1000,
                effect: $scope.slider.effect
            };
            $scope.loadVideos(false);
        };

        $scope.doRefresh = function (action) {
            $scope.refreshPaginationData();
            $scope.videos = [];
            $scope.refresh = true;

            switch (action) {
                case 'loadMainPageContents':
                    $scope.loadMainPageContents(true);
                    break;
                case 'loadVideos':
                    $scope.loadVideos(true);
                    break;
                case 'loadVideo':
                    $scope.loadVideo(true);
                    break;
                default:
                    break;
            }
        };

        $scope.loadMore = function (action) {
            // console.log('load more!');
            if (!$scope.moreItems) {
                $scope.$broadcast('scroll.infiniteScrollComplete');
                return;
            }

            $scope.nextPage++;
            switch (action) {
                case 'loadVideos':
                    $scope.loadVideos(false);
                    break;
                default:
                    break;
            }
        };

        $scope.loadVideo = function (refresh) {
            // Load videos
            if (isOnline()) {
                if (!refresh) {
                    $scope.loading = true;
                }
                $http({
                    method: 'GET',
                    noLoading: true,
                    url: $scope.initData.get_video_url + '/' +
                        $scope.initData.sl + '?' +
                        'code=' + $scope.video.code
                })
                    .success(function (data) {
                        if (data.success) {
                            $scope.video = data.video;
                            localStorage.save(data.video, 'current-video');
                        } else {
                            $scope.storage = lang('video-commerce.fail_to_load');
                        }
                        $scope.loading = false;
                        $scope.refresh = false;
                        $scope.$broadcast('scroll.infiniteScrollComplete');
                        $scope.$broadcast('scroll.refreshComplete');
                        $scope.$broadcast('pzz_video.video_load_finished');
                        $scope.$emit('pzz_video.video_load_finished');
                    })
                    .error(function () {
                        $scope.storage = lang('video-commerce.fail_to_load');
                        $scope.loading = false;
                        $scope.refresh = false;
                        $scope.$broadcast('scroll.infiniteScrollComplete');
                        $scope.$broadcast('scroll.refreshComplete');
                    });
            } else {
                $scope.loading = false;
                $scope.refresh = false;
                checkYourInternet();
            }
        };

        $scope.loadVideos = function (refresh) {
            var cat_id_query = $stateParams.cat_id === undefined ? 'null=null' : 'cat_id=' + $stateParams.cat_id;
            // Load videos
            if (isOnline()) {
                if (!refresh) {
                    $scope.loading = true;
                }
                $http({
                    method: 'GET',
                    noLoading: true,
                    url: $scope.initData.get_videos_list_url + '/' +
                        $scope.initData.sl + '?' +
                        cat_id_query + '&' +
                        'request_type=json&' +
                        'page=' + $scope.nextPage
                })
                    .success(function (data) {
                        if (data.videos) {
                            for (var i = 0; i < data.videos.length; i++) {
                                $scope.videos.push(data.videos[i]);
                            }
                            if (data.videos.length < 6) {
                                $scope.moreItems = false;
                            } else {
                                $scope.moreItems = true;
                            }
                        } else {
                            $scope.storage = lang('video-commerce.fail_to_load');
                        }
                        $scope.loading = false;
                        $scope.refresh = false;
                        $scope.$broadcast('scroll.infiniteScrollComplete');
                        $scope.$broadcast('scroll.refreshComplete');
                    })
                    .error(function () {
                        $scope.storage = lang('video-commerce.fail_to_load');
                        $scope.loading = false;
                        $scope.refresh = false;
                        $scope.$broadcast('scroll.infiniteScrollComplete');
                        $scope.$broadcast('scroll.refreshComplete');
                    });
            } else {
                $scope.loading = false;
                $scope.refresh = false;
                checkYourInternet();
            }

        };

        $scope.handleVideoClick = function (item) {
            // console.log('hi click', item);
            localStorage.save(item, 'current-video');
            $scope.loadVideo(false);
            $location.path(item.link);
        };

        $scope.handlePurchasedVideosClick = function () {
            // /nav/video-commerce869/purchasedVideos
            $location.path('/nav/' + $scope.initData.slug + '/purchasedVideos');
        };

        $scope.handleVideoPlay = function (item) {
            // console.log('hey play me', item);
            $scope.playVideo(item.video, item);
        };

        $scope.handleTeaserVideoPlay = function (item) {
            // console.log('hey play me', item);
            $scope.playVideo(item.teaser, item);
        };

        $scope.handleDownloadVideo = function (item) {
            if (isOnline()) {
                $http({
                        method: 'GET',
                        noLoading: true,
                        url: $scope.initData.get_video_download_url + '/' +
                            $scope.initData.sl + '?' +
                            'video_id=' + item.video_id + '&' +
                            'link_to_download=' + item.video
                    })
                    .success(function (data) {
                        // console.log(data);
                        if (data.success) {
                            $scope.download(data.url, item);
                        } else {
                            showToast(data.message ? data.message : lang('video-commerce.fail_to_download'));
                        }
                        $scope.loading = false;
                        $scope.$broadcast('scroll.refreshComplete');
                    })
                    .error(function () {
                        $scope.storage = lang('video-commerce.fail_to_download');
                        $scope.loading = false;
                        $scope.$broadcast('scroll.refreshComplete');
                    });
            } else {
                $scope.loading = false;
                checkYourInternet();
            }
        };

        $rootScope.downloader = {};
        $scope.download = function (url, item) {
            var orginalVideoUrl = item.video;
            if (!isDevice && $scope.config.download_in_app === true) {
                return;
            }
            if (isDevice && $scope.config.download_in_app !== true && ionic.Platform.isIOS()) {
                cordova.InAppBrowser.open(url, '_system');
            }
            if (typeof $rootScope.downloader[item.id] !== 'undefined' && Object.keys($rootScope.downloader[item.id]).length) {
                item = $rootScope.downloader[item.id];
            }
            if (typeof item.fileTransfer !== 'undefined') {
                item.fileTransfer.abort();
                item.downloading = false;
                return;
            }
            if (typeof item.downloading == 'undefined' || item.downloading == false) {
                if (typeof FileTransfer !== "undefined") {
                    url = item.video || item.teaser || url;
                    var startDownload = function () {
                        $rootScope.downloader[item.id] = item;
                        item.downloading = true;
                        try {
                            item.fileTransfer = new FileTransfer();
                            item.fileTransfer.download(
                                encodeURI(decodeURIComponent(url)), // url
                                path, // where to store file
                                function (entry) { // success download
                                    showToast(lang('video-commerce.download_success'));
                                    addDownloadedVideos(url, path);
                                    item = $rootScope.downloader[item.id];
                                    $scope.$apply(function () {
                                        item.downloaded = true;
                                        item.downloading = false;
                                        item.video = entry.toURL();
                                    });
                                    delete $rootScope.downloader[item.id];
                                },
                                function (error) { // fail download
                                    if (error.code !== 4) {
                                        showToast(lang('video-commerce.fail_to_download'), 3000, 'center');
                                    }
                                    $scope.$apply(function () {
                                        item.video = orginalVideoUrl;
                                        item.downloading = false;
                                        item.downloaded = false;
                                    });
                                    delete $rootScope.downloader[item.id];
                                },
                                true
                            );
                            item.fileTransfer.onprogress = function (progressEvent) {
                                if (progressEvent.lengthComputable) {
                                    var persentage = Number.parseFloat(progressEvent.loaded / progressEvent.total * 100).toFixed(1);
                                    if (persentage > 5) {
                                        item.video = path; // user can play video on downloading 
                                    }
                                    if (persentage >= 100) {
                                        persentage = false;
                                    }
                                    $scope.$apply(function () {
                                        item.downloadProgress = persentage;
                                    });
                                }
                            };
                            return;
                        } catch (error) {
                            item.video = orginalVideoUrl;
                            item.downloading = false;
                            item.downloaded = false;
                        }
                    };
                    var path;
                    if ($scope.config.download_in_app) {
                        path = cordova.file.dataDirectory + '/' + hashCode(url);
                        startDownload();
                    } else {
                        path = cordova.file.externalRootDirectory + '/download/' + item.title + get_url_extension(url);
                        cordova.plugins.permissions.requestPermission(
                            cordova.plugins.permissions.WRITE_EXTERNAL_STORAGE,
                            function (status) {
                                if (status.hasPermission) {
                                    startDownload();
                                } else {
                                    showToast(lang('video-commerce.permission_help'), 5000);
                                }
                            },
                            function () {
                                showToast(lang('video-commerce.permission_help'), 5000);
                            }
                        );
                    }        
                } else {
                    window.open(url, '_system');
                }
            }
        };

        function get_url_extension(url) {
            return '.' + url.split(/[#?]/)[0].split('.').pop().trim();
        }

        function addDownloadedVideos(videoUrl, localpath) {
            var data = {};
            data[hashCode(videoUrl)] = localpath;
            localStorage.add(data, 'downloaded_videos');
        }
        function removeDownloadedVideos(videoUrl) {
            localStorage.remove('downloaded_videos' , hashCode(videoUrl));
        }

        $scope.playVideo = function (videoUrl, item) {
            // for inapp link
            if (item.type == 'link') {
                window.location.hash = videoUrl;
                return;
            }

            if (item.type == 'music') {
                if (item.hasOwnProperty('localPath')) {
                    videoUrl = item.localPath;
                    window.resolveLocalFileSystemURL(
                        item.localPath,
                        function () { // file exist
                        },
                        function () { // file not found
                            item.downloaded = false;
                            removeDownloadedVideos(item.video);
                            $rootScope.playingMusic = {
                                url: videoUrl,
                                title: item.title,
                                keepPlaying: true,
                                itemId: item.id,
                                videoId: item.video_id
                            };
                            $scope.togglePlayback = !$scope.togglePlayback;
                        }
                    );
                }
                if ($scope.playInProgress) {
                    var vid = document.getElementById("web-app-video");
                    var source = vid.getElementsByTagName('source')[0];
                    vid.pause();
                    $scope.playInProgress = false;
                }
                if (typeof $rootScope.playingMusic != "undefined" && typeof $rootScope.playingMusic.url !== "undefined" && $rootScope.playingMusic.url == videoUrl) {
                    $scope.togglePlayback = !$scope.togglePlayback;
                    return;
                }
                $scope.showMusicPlayer = true;
                $rootScope.playingMusic = {
                    url: videoUrl,
                    title: item.title,
                    keepPlaying: true,
                    itemId: item.id,
                    videoId: item.video_id
                };

                $timeout(function () {
                    $scope.togglePlayback = !$scope.togglePlayback;
                }, 100);

                if ($scope.config.continue_from_last_position) {
                    var savedPosition = localStorage.get('musics-position', item.id);
                    if (savedPosition > 60) {
                        $ionicPopup.confirm({
                            title: lang('video-commerce.continue_last_playing'),
                            template: lang('video-commerce.continue_last_playing_message'),
                            cancelText: lang('video-commerce.start_over'),
                            okText: lang('video-commerce.continue')
                        }).then(function (res) {
                            if (res) {
                                MediaManager.seekTo(savedPosition);
                            }
                        });
                    }
                }                

                $ionicScrollDelegate.resize();
                $ionicScrollDelegate.scrollTop(true);
                return;
            } else {
                MediaManager.stop();
                $scope.showMusicPlayer = false;
            }

            // in web app        
            if (!isDevice) {
                $(".product-single-image, #playButton").hide();
                var html = '<video id="web-app-video" width="auto" height="auto" controls controlsList="nodownload">';
                html += '<source src="' + videoUrl + '" type="video/mp4">';
                html += 'Your browser does not support HTML5 video.';
                html += '</video>';

                if ($scope.playInProgress) {
                    var vid = document.getElementById("web-app-video");
                    var source = vid.getElementsByTagName('source')[0];
                    vid.pause();
                    if (source.src !== videoUrl) {
                        source.src = videoUrl;
                        vid.load();
                        vid.play();
                    }
                    $scope.playInProgress = false;
                    return;
                }

                if (!$scope.videoHtmlAdded) {
                    $("#videoContainer").after(html);
                    var vid = document.getElementById("web-app-video");
                    vid.play();
                    $scope.videoHtmlAdded = true;
                }

                var vid = document.getElementById("web-app-video");
                vid.loop = false;
                $scope.playInProgress = true;
                $ionicScrollDelegate.resize();
                $ionicScrollDelegate.scrollTop(true);               
                return;
            }


            // in app
            var options = {
                successCallback: function () {},
                errorCallback: function (errMsg) {
                    showError(lang('video-commerce.video_is_not_playable_or_deleted'))
                },
                orientation: 'landscape',
                shouldAutoClose: true,
                controls: true
            };

            if (item.hasOwnProperty('localPath')) {
                window.resolveLocalFileSystemURL(
                    item.localPath,
                    function () { // file exist
                        window.plugins.streamingMedia.playVideo(item.localPath, options);
                    },
                    function () { // file not found
                        item.downloaded = false;
                        removeDownloadedVideos(item.video);
                        window.plugins.streamingMedia.playVideo(videoUrl, options);
                    }
                );
            } else {
                window.plugins.streamingMedia.playVideo(videoUrl, options);
            }
        };

        $scope.$watch('showMusicPlayer', function (newValue) {
            newValue ? $('.imageContiner').hide() : $('.imageContiner').show();
        });

        var MEDIA_RUNNING = 2;
       
        if ($rootScope.playingMusic && $rootScope.playingMusic.status == MEDIA_RUNNING && $rootScope.playingMusic.videoId == $scope.video.id) {
            $('.imageContiner').hide();
        }

        $rootScope.$watch('playingMusic.status', function (newValue, oldValue) {
            if (newValue == MEDIA_RUNNING) {
                if (angular.isDefined($rootScope.saveCurrentMusicTime)) {
                    $interval.cancel($rootScope.saveCurrentMusicTime);
                }
                $rootScope.saveCurrentMusicTime = $interval(function () {
                    var data = {};
                    data[$rootScope.playingMusic.itemId] = $rootScope.playingMusic.progress;
                    localStorage.add(data, 'musics-position');
                }, 5000);
            } else {
                if (angular.isDefined($rootScope.saveCurrentMusicTime)) {
                    $interval.cancel($rootScope.saveCurrentMusicTime);
                    $rootScope.saveCurrentMusicTime = undefined;
                }
            }
        });

        $scope.forward = function () {
            MediaManager.seekTo($rootScope.playingMusic.progress + 15);
        };

        $scope.backward = function () {
            MediaManager.seekTo($rootScope.playingMusic.progress - 15);
        };

        $scope.handlePayButton = function (item) {
            // console.log('hello pay', item, $scope.initData);
            if (!$rootScope.isUserLoggedIn()) {
                $rootScope.forceToLoggedIn();
                return;
            }

            if (isOnline()) {
                var windowReference = window;
                $http({
                    method: 'POST',
                    noLoading: false,
                    url: url + '/api/v1/widget/post/video-commerce/postCheckout?sl=' + $scope.initData.sl + '&request_type=json',
                    data: {
                        videoId: item.code.split("|")[1]
                    }
                })
                    .success(function (data) {
                        // console.log(data);
                        if (typeof data === 'undefined') {
                            return;
                        }
                        if (typeof data.error !== 'undefined' && data.error !== '') {
                            showError(data.error);
                            return;
                        }
                        var provider = data.provider;
                        if (provider != 'cafebazaar' && data.paymenturl !== '') {
                            if (isDevice) {
                                cordova.InAppBrowser.open(encodeURI(data.paymenturl), '_system');
                            } else if (ionic.Platform.isIOS()) {
                                window.location = encodeURI(data.paymenturl);
                            } else {
                                // set cookie for back button in web applications
                                var d = new Date();
                                d.setTime(d.getTime() + (3600000));
                                var expires = "expires=" + d.toUTCString();
                                document.cookie = "backUrl=" + window.location + ";" + expires + ";path=/";
                                window.open(data.paymenturl, '_system');
                            }
                        } else if (provider === 'cafebazaar') {
                            if (!isDevice) {
                                showWarning(lang('video-commerce.not_available_in_similator'));
                                return {
                                    data: {
                                        success: false,
                                        error: 'This feature is only available in application',
                                        payload: {
                                            provider: 'cafebazaar',
                                        },
                                    },
                                };
                            }
                            var type = 'buy';
                            var appUserDataId = data.appUserDataId;
                            $ionicLoading.show();
                            try {
                                inappbilling.init(
                                    function () {
                                        inappbilling[type](
                                            function (result) {
                                                $scope.verifyInappPlanPayment(result, type, appUserDataId);
                                            },
                                            function () {
                                                showError(lang('video-commerce.payment_failed'));
                                                return;
                                            },
                                            $scope.video.cafebazaar.product_id
                                        );
                                    },
                                    function () {
                                        showWarning(
                                            lang('video-commerce.fail_iap_message1') +
                                            $scope.initData.payment_provider +
                                            lang('video-commerce.fail_iap_message2')
                                        );
                                        setTimeout(function () {
                                            window.open('bazaar://login', '_system');
                                        }, 3000);
                                    }, {
                                    showLog: false
                                },
                                    'com.farsitel.bazaar',
                                    'ir.cafebazaar.pardakht.InAppBillingService.BIND',
                                    $scope.initData.cafebazaar.RSAKey
                                );
                            } catch (error) {
                                $scope.loading = false;
                                $ionicLoading.hide();
                                $scope.$broadcast('scroll.refreshComplete');
                                showWarning(
                                    lang('video-commerce.fail_iap_confirm') +
                                    $scope.initData.payment_provider
                                );
                            }
                        }

                        $scope.$broadcast('scroll.refreshComplete');
                    })
                    .error(function () {
                        $scope.storage = lang('video-commerce.payment_failed');
                        $scope.loading = false;
                        $scope.$broadcast('scroll.refreshComplete');
                    });
            } else {
                $scope.loading = false;
                checkYourInternet();
            }
        };

        $scope.planInappPaymentSucceeded = function (result) {
            if (result.success) {
                $ionicLoading.hide();
                $scope.video.is_purchased = true;
                $ionicPopup.show({
                    cssClass: 'iap_pupop',
                    title: lang('video-commerce.payment_successed'),
                    buttons: [{
                        text: lang('global.close'),
                        type: 'button-clear button-balanced'
                    }],
                });
                window.location.hash = window.location.hash;
            } else {
                $ionicLoading.hide();
                $ionicPopup.show({
                    cssClass: 'iap_pupop',
                    title: lang('video-commerce.the_purchase_failed'),
                    buttons: [{
                        text: lang('global.close'),
                        type: 'button-clear button-balanced'
                    }],
                });
            }
        };

        $scope.verifyInappPlanPayment = function (result, type, appUserDataId) {
            $ionicLoading.show();
            $http({
                    method: 'POST',
                    url: url + '/api/v1/widget/post/video-commerce/cafebazaarVerifyPlanPayment?sl=' + $scope.initData.sl,
                    data: {
                        packageName: result.packageName,
                        productId: result.productId,
                        purchaseToken: result.purchaseToken,
                        provider: 'cafebazaar',
                        code: result.code,
                        type: type,
                        appUserDataId: appUserDataId
                    },
                    responseType: 'json'
                })
                .success(function (data) {
                    $ionicLoading.hide();
                    if (data.success === true) {
                        $scope.planInappPaymentSucceeded(data);
                    } else {
                        showError(data.error);
                    }
                }).error(function () {
                    $ionicLoading.hide();
                    $ionicPopup.alert({
                        cssClass: 'iap_pupop',
                        title: '',
                        template: lang('video-commerce.fail_iap_confirm2'),
                        buttons: [{
                            text: '<b>' + lang('video-commerce.try_again') + '</b>',
                            type: 'button-positive',
                            onTap: function () {
                                $scope.verifyInappPlanPayment(result, type, appUserDataId);
                            }
                        }, {
                            text: lang('global.close'),
                            type: 'button-stable'
                        }]
                    });
                });
        };

        $scope.loadMainPageContents = function (refresh) {
            // Load
            if (isOnline()) {
                if (!refresh) {
                    $scope.loading = true;
                }
                $http({
                    method: 'GET',
                    noLoading: true,
                    url: url + '/api/v1/widget/route/video-commerce/getVideosAndLanding/' + $scope.initData.sl
                })
                    .success(function (data) {
                        if (data.landing) {
                            $scope.landing = data.landing;
                            localStorage.save(data.landing.categories, 'current-categories');
                            localStorage.save(data.landing.slider, 'current-slider');
                            var loop = $scope.landing.slider.effect == 'cube' ? false : true;
                            $scope.slideroptions = {
                                preventClicks: false,
                                preventClicksPropagation: false,
                                autoplayDisableOnInteraction: false,
                                loop: loop,
                                speed: 400,
                                //autoHeight: true,
                                autoplay: $scope.landing.slider.duration * 1000,
                                effect: $scope.landing.slider.effect
                            };

                            if (data.landing.categories_count <= 1) {
                                $scope.loadVideos(false);
                            }

                            setTimeout(function () {
                                $ionicScrollDelegate.scrollTo(2000, 0, false);
                            }, 100);
                        } else {
                            $scope.storage = lang('video-commerce.fail_to_load');
                        }
                        $scope.loading = false;
                        $scope.$broadcast('scroll.refreshComplete');
                    })
                    .error(function () {
                        $scope.storage = lang('video-commerce.fail_to_load');
                        $scope.loading = false;
                        $scope.$broadcast('scroll.refreshComplete');
                    });
            } else {
                $scope.loading = false;
                checkYourInternet();
            }
        };

        $scope.handleDownloadTeaser = function (item) {
            if (!isOnline()) {
                $scope.loading = false;
                checkYourInternet();
                return;
            }
            $http({
                method: 'GET',
                noLoading: true,
                url: $scope.initData.get_teaser_download_url + '/' +
                    $scope.initData.sl + '?' +
                    'link_to_download=' + item.teaser
            })
                .success(function (data) {
                    if (data.success) {
                        $scope.download(data.url, item);
                    } else {
                        $scope.storage = lang('video-commerce.fail_to_download');
                    }
                    $scope.loading = false;
                    $scope.$broadcast('scroll.refreshComplete');
                })
                .error(function () {
                    $scope.storage = lang('video-commerce.fail_to_download');
                    $scope.loading = false;
                    $scope.$broadcast('scroll.refreshComplete');
                });
        };
    });


function hashCode(string) {
    var hash = 0, i, chr;
    string = string + ''; //convert to string
    for (i = 0; i < string.length; i++) {
        chr = string.charCodeAt(i);
        hash = ((hash << 5) - hash) + chr;
        hash |= 0; // Convert to 32bit integer
    }
    return ('v' + hash).replace('-', 'm');
}