(function (angular) {
    'use strict';
 
    function PZZLessonListController($scope, $rootScope, localStorage) {
        var ctrl = this;

        this.$onChanges = function () {
            _setVariables();
        };

        this.$onInit = function () {
            _setVariables();
            _setDownloadStatus();
            $scope.videos = ctrl.videos;
            ctrl.onPlayVideo = function (item) {
                if (ctrl.isPlayable(item)) {
                    ctrl.handlePlayBtn()(item);
                }
            };

            ctrl.onDownloadVideo = function (index) {
                if (ctrl.canDownload && ctrl.isPlayable($scope.videos[index])) {
                    ctrl.handleDownloadBtn()($scope.videos[index]);
                }
            };

            ctrl.isPlayable = function (item) {
                return (!ctrl.itemDisabled || !item.locked);
            };
        };

        function _setVariables() {
            ctrl.itemDisabled = ctrl.isClickable !== undefined && !ctrl.isClickable;
            ctrl.canDownload = ctrl.isDownloadable !== undefined && ctrl.isDownloadable == 1;

        }

        function _setDownloadStatus() {
            var downloaded_videos = localStorage.all('downloaded_videos');
            var code;
            if (Object.keys(downloaded_videos).length !== 0) {
                for (var i in ctrl.videos) {
                    code = hashCode(ctrl.videos[i].video);
                    if (typeof downloaded_videos[code] !== 'undefined') {
                        ctrl.videos[i].downloaded = true;
                        ctrl.videos[i].localPath = downloaded_videos[code];
                    }
                    if ($rootScope.downloader.hasOwnProperty(ctrl.videos[i].id)) {
                        ctrl.videos[i] = $rootScope.downloader[ctrl.videos[i].id];
                    }
                }
            }
        }
    }

    angular.module('ngApp').component('pzzLessonList', {
        bindings: {
            videos: '<',
            handlePlayBtn: '&',
            handleDownloadBtn: '&',
            isClickable: '<',
            isDownloadable: '<'
        },
        controller: PZZLessonListController,
        template: 
        '   <ion-item   '  + 
        '       ng-repeat="entry in videos track by $index" '  + 
        '       class="item item{{ $index }} item-contact lessons" ' +
        '       ng-class="{\'item-icon-left padding-right\' : $ctrl.canDownload}" ' +
        '       id="{{ entry.id }}"' +
        '   > '  + 
        '       <div ng-if="$ctrl.isPlayable(entry)" ng-click="$ctrl.onPlayVideo(entry)" class="clickable-aria"></div> ' +
        '       <div class="play-button" ng-class="{\'play-video\' : $ctrl.isPlayable(entry), \'lock \' : !$ctrl.isPlayable(entry), \'pause\' : entry.playInProgress}"></div>' +
        '       <h2 class="title"> {{ ::entry.title }} </h2>' +
        '       <span ng-if="entry.duration" class="item-note"> {{ entry.duration | secondsToHMS | persianNumber }} <i class="ion-android-time"></i></span>' +
        '       <i ng-if="$ctrl.canDownload" ng-click="$ctrl.onDownloadVideo($index)" class="icon ion-ios-cloud-download-outline" ng-class="{\'ion-ios-loop looping\': entry.downloading, \'ion-ios-checkmark-outline downloaded\': entry.downloaded, \'disabled\': $ctrl.itemDisabled}"></i>'+
        '       <span class="download-progress" ng-show="entry.downloadProgress">{{ entry.downloadProgress | persianNumber }}%</span>' +
        '        '  + 
        '   </ion-item> '
    });
})(window.angular);