(function (angular) {
    'use strict';
 
    function PZZVideoController ($scope, $element, $attrs, $sce) {
        var ctrl = this;

        this.$onChanges = function () {
            _setVariables();
        };

        this.$onInit = function () {
            _setVariables();

            ctrl.setTab = function (newTab) {
                ctrl.tab = newTab;
            };
        
            ctrl.isSet = function (tabNum) {
                return ctrl.tab === tabNum;
            };

            ctrl.onPayButton = function (item) {
                ctrl.handlePayButton()(item);
            };

            ctrl.onTeaserPlayClick = function (item) {
                ctrl.handleTeaserPlayButton()(item);
            };

            ctrl.onDownloadVideo = function (item) {
                if (!ctrl.canDownload) {
                    return;
                }
                ctrl.handleDownloadBtn()(item);
            };
        };

        this.trustedHtml = function (plainText) {
            return $sce.trustAsHtml(plainText);
        };

        function _setVariables() {
            ctrl.hasTabs = ctrl.video.tabs != null;
            ctrl.hasTeaser = ctrl.video.has_teaser == true;
            ctrl.tab = 1;
            ctrl.canDownload = ctrl.isDownloadable !== undefined && ctrl.isDownloadable == 1;
            ctrl.isDevice = isDevice;
        }
    }

    angular.module('ngApp').component('pzzVideo', {
        bindings: {
            video: '<',
            handlePayButton: '&',
            handleTeaserPlayButton: '&',
            handleDownloadBtn: '&',
            isDownloadable: '<',
            widgetType: '<',
        },
        controller: PZZVideoController,
        template: 
        '       <div class="imageContiner" ng-if="$ctrl.video.image">  '  + 
        '           <div style="position: relative;">  '  + 
        '               <div id="videoContainer" ng-if="!$ctrl.isDevice"></div>'  +
        '               <svg id="playButton" ng-if="$ctrl.hasTeaser" ng-click="$ctrl.onTeaserPlayClick($ctrl.video)" style="position: absolute; height: 30%; width: 30%; top: 45%; left: 45%; background-position: center; background-repeat: no-repeat; z-index: 2; fill: white; opacity: 0.65; -webkit-transform: translate(-35%, -50%); -ms-transform: translate(-35%, -50%); transform: translate(-35%, -50%);" class="video-overlay-play-button" viewBox="0 0 200 200" alt="Play video"> <circle cx="100" cy="100" r="90" fill="#2e2e2e" stroke-width="15" stroke="#000"/> <polygon points="70, 55 70, 145 145, 100" fill="white"/> </svg>' +
        '               <img class="product-single-image"   '  + 
        '                   ng-src="{{ $ctrl.video.image }}"   '  + 
        '               >  '  + 
        '           </div>  '  + 
        '       </div>  '  + 
        '         '  + 
        '       <div class="content {{$ctrl.hasTabs ? \'\' : \'\'}} bg-primary image-viewer">  ' +
        '           <h4 class="header-product title item_name item-contact transparent">  '  + 
        '               {{$ctrl.video.title}}'  + 
        '               <i ng-if="$ctrl.canDownload && $ctrl.widgetType == \'video\'" class="icon ion-ios-cloud-download-outline" style="font-size: 2rem; position: relative; float: left;" ng-click="$ctrl.onDownloadVideo($ctrl.video)"></i> '  + 
        '               <div ng-if="$ctrl.video.duration && $ctrl.video.lessons_count > 0" style="border-top: 1.5px solid #ccc; margin-top: 11px; border-bottom: 1.5px solid #ccc; padding-top: 11px;">' +
        '                   <div style="display: inline-block; width: 100%; padding: 0 29px;" class="ng-binding"> ' +
        '                       <div style=" float: right;" class="product-description"> ' + 
        '                           <div style="font-size: 30px; color: #adadad; height: 43px;"> ' + 
        '                               <i class="icon ion-ios-clock-outline"></i> '  +
        '                           </div> ' +
        '                           <div style="font-weight: 100; font-size: 13px;"> ' + lang('video-commerce.duration') + ' </div> ' +
        '                           <div style="font-size: 15px; font-weight: 600;"> {{ $ctrl.video.duration | secondsToHMS  | persianNumber }}</div> ' +
        '                       </div> ' +
        '                       <div style="float: left;" class="product-description">  ' +
        '                           <div style="font-size: 30px; color: #adadad; height: 43px;"> ' + 
        '                               <i class="icon ion-ios-list-outline"></i> ' +
        '                           </div> ' +
        '                           <div style="font-weight: 100; font-size: 13px;">' + lang('video-commerce.video_count') + ' </div> ' +
        '                           <div style="font-size: 15px; font-weight: 600;"> {{ $ctrl.video.lessons_count | persianNumber }} </div> ' +
        '                       </div> ' + 
        '                   </div>' +
        '               </div>' +
        '               <button style="background-color: #4ECE3D; border-radius: 3px; border: 1px solid #40b630; color: #ffffff; cursor: pointer; -webkit-transition: all 0.2s; transition: all 0.2s; background-image: unset; color: white; font-family: inherit;" ng-if="!$ctrl.video.is_purchased" class="button button-full icon-center ion-android-cart item_add button-cart" ng-click="$ctrl.onPayButton($ctrl.video)">  ' +
                            lang('video-commerce.checkout') + ' ({{ $ctrl.video.price | toman }})  '  + 
        '               </button>  '  + 
        '           </h4>  '  + 
        '           <div ng-if="!$ctrl.hasTabs && $ctrl.video.description" class="tabs-contents item-contact transparent">  '  + 
        '               <div ng-bind-html="$ctrl.trustedHtml($ctrl.video.description)"></div>  '  + 
        '           </div>  '  + 
        '           <div ng-if="$ctrl.hasTabs">  '  + 
        '               <div class="tabs-continer">  '  + 
        '                   <div class="tabs tabs-striped tabs-top">  '  + 
        '                       <a class="tab-item active" ng-click="$ctrl.setTab(1)" ng-class="{ active: $ctrl.isSet(1) }">  '  + 
        '                           ' + lang('video-commerce.description') + '  ' +
        '                       </a>  '  + 
        '                       <a class="tab-item" ng-repeat="tab in $ctrl.video.tabs track by $index" ng-click="$ctrl.setTab($index+2)" ng-class="{ active: $ctrl.isSet($index+2) }">  '  + 
        '                           {{ tab.title }}  '  + 
        '                       </a>  '  + 
        '                   </div>  '  + 
        '                   <div class="tabs-contents item-contact transparent">  '  + 
        '                       <div ng-show="$ctrl.isSet(1)">  '  + 
        '                           <div ng-bind-html="$ctrl.video.description"></div>  '  + 
        '                       </div>  '  + 
        '                       <div ng-repeat="tab in $ctrl.video.tabs track by $index" ng-show="$ctrl.isSet($index+2)">  '  + 
        '                           <div ng-bind-html="tab.body"></div>  '  + 
        '                       </div>  '  + 
        '                   </div>  '  + 
        '               </div>  '  + 
        '           </div>  '  + 
        '      </div>  '
    });
})(window.angular);